#include	"stdThread.h"

#ifdef _MSC_VER			// Windowsのとき
#define DISABLE_C4996   __pragma(warning(push)) __pragma(warning(disable:4996))
#define ENABLE_C4996    __pragma(warning(pop))
#else					// Linuxのとき
#define DISABLE_C4996
#define ENABLE_C4996
#endif

// メニューで使用する文字
#define		CMD_QUIT_CHAR		'q'
#define		CMD_SEND_MSG_CHAR	'm'

// 関数の宣言
BOOL CreateAndConnectSocket(LPCSTR szAddress, WORD wPort);	// ソケットの作成と接続処理
BOOL DoSend(char *pcData, int iSize);			// 送信処理の実行
BOOL DestroySocket(SOCKET &fd);					// 切断とソケットの破棄
void Stop();									// すべてのソケットを破棄する
void DispMenu();								// メニューの表示
int	 GetKeyString(LPSTR pszString, int iSize);	// キーボード入力文字の取得

// 変数の宣言
SOCKET		m_fdClient;							// 接続、通信用ソケット


int main(int argc, char *argv[])
{
	char	szKeyInBuff[81];

	m_fdClient = INVALID_SOCKET;
	// 起動パラメータチェック
	if (argc != 3)
	{
		fprintf(stderr, "Usage:  %s <ServerAddress> <ServerPort>\n", argv[0]);
		goto L_END;
	}
	// ソケットの作成と接続処理
	// m_fdClientが接続済み状態になる
	if (CreateAndConnectSocket(argv[1], (WORD)atol(argv[2])) == FALSE)
		goto L_END;
	// メニューの表示
	DispMenu();
	while (1)
	{
		// キーボードから入力された文字列が'q'なら終了
		// 'm'ならメッセージを入力後は送信する		
		switch (GetKeyString(szKeyInBuff, sizeof(szKeyInBuff)))
		{
			case 0:							// 入力なし、何もしない
				break;
			case CMD_QUIT_CHAR:				// 'q'入力終了
				goto L_END;
			case CMD_SEND_MSG_CHAR:			// 'm'入力szKeyInBuffに文字列が格納されている
				DoSend(szKeyInBuff, strlen(szKeyInBuff));
				// メニューの表示
				DispMenu();
				break;
		}
	}

L_END:
	// 切断とすべてのソケットの破棄
	Stop();
	return(0);
}


//----------------------------------------------
// function
//  TCPソケットの作成, connectの実行
//  接続成功したときソケットはm_fdClientに格納する
// parameter
//  LPCSTR       szAddress      [in]接続先アドレス
//  WORD         wPort          [in]接続先ポート
// return
//  TRUE/FALSE
//----------------------------------------------
BOOL CreateAndConnectSocket(LPCSTR szAddress, WORD wPort)
{
	BOOL		fRet = TRUE;
	SOCKET      fd;
	struct addrinfo		hints, *pres = NULL, *pTemp = NULL;
	char		szPort[NI_MAXSERV];

	memset(&hints, 0, sizeof(hints));
	hints.ai_socktype = SOCK_STREAM;		// TCP
	DISABLE_C4996
	sprintf(szPort, "%d", wPort);			// ポート番号=サービス
	ENABLE_C4996
	// アドレスを指定してgetaddrinfoを実行 接続先アドレス情報を得る
	if (getaddrinfo(szAddress, szPort, &hints, &pres) == 0)
	{
		pTemp = pres;
		while (pTemp != NULL)
		{
			// socket, connectを実行
			if ((fd = socket(pTemp->ai_family, pTemp->ai_socktype, pTemp->ai_protocol)) == INVALID_SOCKET)
				goto L_NEXT;
			if (connect(fd, pTemp->ai_addr, (int)pTemp->ai_addrlen) == SOCKET_ERROR)
			{
				fprintf(stderr, "Err:connect %d\n", errno);
				DestroySocket(fd);
				goto L_NEXT;
			}
			// 接続成功したのでこのソケット採用することにして抜ける
			fRet = TRUE;
			break;
		L_NEXT:
			pTemp = pTemp->ai_next;
		}
		freeaddrinfo(pres);
	}
	if (fd == INVALID_SOCKET)	// 接続に失敗
	{
		fprintf(stderr, "Err:connect %d\n", errno);
		fRet = FALSE;
		goto L_END;
	}
	m_fdClient = fd;			// 接続済み状態のソケットをm_fdClientに格納

L_END:
	return(fRet);
}

//----------------------------------------------
// function
//    sendの実行
// parameter
//    char		*pcData		[in]送信データ
//    int		iSize		[in]データ数
// return
//    TRUE/FALSE
//----------------------------------------------
BOOL DoSend(char *pcData, int iSize)
{
	BOOL		fRet = TRUE;
	fd_set      wfds;
	struct timeval   tv;

	while (1)
	{
		tv.tv_sec = 0;
		tv.tv_usec = 10 * 1000;							// 10msec
		FD_ZERO(&wfds);
		FD_SET(m_fdClient, &wfds);						// 送信可能か監視
		select(m_fdClient + 1, NULL, &wfds, NULL, &tv);	// タイムアウトまでSleepと同等
		if (FD_ISSET(m_fdClient, &wfds))				// 送信可能ならsend実施
		{
			if (send(m_fdClient, pcData, iSize, 0) != iSize)
			{
				fprintf(stderr, "Err:send %d\n", errno);
				fRet = FALSE;
			}
			break;
		}
	}
	return(fRet);
}

//----------------------------------------------
// function
//    TCPソケットの破棄
// parameter
//    SOCKET		&fd[in/out]破棄するソケット
// return
//    TRUE/FALSE
//----------------------------------------------
BOOL DestroySocket(SOCKET &fd)
{
	fprintf(stderr, "DestroySocket()\n");
	if (fd != INVALID_SOCKET)
	{
		shutdown(fd, SHUT_RDWR);     // 受信も送信も停止
		close(fd);
		fd = INVALID_SOCKET;
	}
	return(TRUE);
}

//----------------------------------------------
// function
//    切断、ソケットの破棄
// parameter
//    なし
// return
//    なし
//----------------------------------------------
void Stop()
{
	fprintf(stderr, "Stop()\n");
	DestroySocket(m_fdClient);
}

//----------------------------------------------
// function
//    メニューの表示
// parameter
//    なし
// return
//    なし
//----------------------------------------------
void DispMenu()
{
//	fprintf(stderr, "\x1b[1;1H\x1b[0J");
	fprintf(stderr, "%c:quit %c:メッセージ送信 : ", CMD_QUIT_CHAR, CMD_SEND_MSG_CHAR);
}

#if 1
//----------------------------------------------
// function
//    キーボード入力(改行まで)の取得
//		入力がないときはすぐにリターン
// parameter
//    LPSTR		pszString	[in/out]データ格納用バッファ
//    int		iSize		[in]バッファのサイズ
// return
//    0:入力なし CMD_QUIT_CHAR:終了 CMD_SEND_MSG_CHAR:メッセージ送信
//----------------------------------------------
int GetKeyString(LPSTR pszString, int iSize)
{
	BOOL			fRet = FALSE;
	int				iRet = 0;

	if (kbhit())
	{
		switch (getchar())
		{
			case CMD_QUIT_CHAR:
				iRet = CMD_QUIT_CHAR;
				break;
			case CMD_SEND_MSG_CHAR:
				fprintf(stderr, "\nInput Msg : ");
				fgets(pszString, iSize, stdin);
				if (strlen(pszString) > 0)
				{
				// 改行まで読み込んでいるので
					if (pszString[strlen(pszString) - 1] == '\n')
						pszString[strlen(pszString) - 1] = 0;
					iRet = CMD_SEND_MSG_CHAR;
				}
				break;
			default:
				break;
		}
	}
	return(iRet);
}
#else
// リモートデバッグではttyじゃないのでkbhitは使えない
int GetKeyString(LPSTR pszString, int iSize)
{
	BOOL			fRet = FALSE;
	int				iRet = 0;
	fd_set			rfds;
	struct timeval	tv;

	tv.tv_sec = 0;
	tv.tv_usec = 1000 * 10;		// 入力がなければ10mecのSleep
	FD_ZERO(&rfds);				// fd_setの初期化
								// 0:標準入力 1:標準出力 2:標準エラー
	FD_SET(0, &rfds);			// 標準入力が調査対象
	select(1, &rfds, NULL, NULL, &tv);
	if (FD_ISSET(0, &rfds))		// ENTERキー入力があった
	{
		memset(pszString, 0, iSize);
		read(0, pszString, iSize);
		switch (pszString[0])
		{
			case CMD_QUIT_CHAR:
				iRet = CMD_QUIT_CHAR;
				break;
			case CMD_SEND_MSG_CHAR:
				fprintf(stderr, "\n送信メッセージを入力してください : ");
				memset(pszString, 0, iSize);
				read(0, pszString, iSize);	// ENTERキーまで読み込む
				if (strlen(pszString) > 0)
				{
				// 改行まで読み込んでいるので
					if (pszString[strlen(pszString) - 1] == '\n')
						pszString[strlen(pszString) - 1] = 0;
					iRet = CMD_SEND_MSG_CHAR;
				}
				break;
			default:
				break;
		}
	}
	return(iRet);
}
#endif



